from .feature_manager import *

if __name__ == "__main__":
    # calc normal features
    # run_normal_features()
    pass
    # calc periodic features
    # TODO: long running, in case really want it uncomment
    # run_periodic_features()
